import urllib3
http = urllib3.PoolManager()


class Lending:
    def __init__(self, url: str):
        self._url = url

        self._id = -1
        self.x, self.y = 0, 0
        self.vx, self.vy = 0, 0
        self.p = 0
        self.l, self.r = 0, 0
        self.state = 'preparing'

        self.last_up = False
        self.last_left = False
        self.last_right = False

    def _base_url(self) -> str:
        return 'http://{0}/'.format(self._url)

    def _request(self, sub_url: str = '', fields=None):
        request = http.request('GET', self._base_url() + sub_url, fields=fields)
        self._update_from_response(request.data)

    def _update_from_response(self, response_data):
        response_data = response_data.decode('ascii').split('\r\n')

        if response_data[0].startswith('Zdar!'):
            self.state = 'success'
            return
        if response_data[0].startswith('Krach!'):
            self.state = 'fail'
            return
        self.state = 'lending'

        self._id = int(response_data[0])
        self.x, self.y = (float(d) for d in response_data[1].split(' '))
        self.vx, self.vy = (float(d) for d in response_data[2].split(' '))
        self.p = float(response_data[3])
        self.l, self.r = (float(d) for d in response_data[4].split(' '))

    def start(self):
        self._request('start')

    def update(self, up: bool = None, left: bool = None, right: bool = None):
        fields = [(b'id', str(self._id).encode('utf-8'))]
        if up is not None:
            self.last_up = up
            fields += [(b'nahoru', str(int(up)).encode('utf-8'))]
        if left is not None:
            self.last_left = left
            fields += [(b'doleva', str(int(left)).encode('utf-8'))]
        if right is not None:
            self.last_right = right
            fields += [(b'doprava', str(int(right)).encode('utf-8'))]

        self._request('stav', fields)

    def __str__(self):
        return 'Lending: {'\
               + 'x: ' + str(self.x) + ', ' \
               + 'y: ' + str(self.y) + ', ' \
               + 'vx: ' + str(self.vx) + ', ' \
               + 'vy: ' + str(self.vy) + ', ' \
               + 'p: ' + str(self.p) + ', ' \
               + 'l: ' + str(self.l) + ', ' \
               + 'r: ' + str(self.r) + ', ' \
               + 'state: ' + str(self.state) \
               + '}'
