from tkinter import *

from auto_lending.lending import Lending


class LendingDrawer(Tk):

    SCALE_X = 4
    SCALE_Y = 4

    WIDTH = 200
    HEIGHT = 200

    TAG_INFO = 'INFO'
    TAG_FLOOR = 'FLOOR'
    TAG_DETAILS = 'DETAILS'
    TAG_SHIP = 'SHIP'
    TAG_RESULT = 'RESULT'

    def __init__(self, lending: Lending):
        super().__init__()
        self.lending = lending

        self.wm_title("LendingDrawer")
        self.configure(background='white')

        self.canvas = Canvas(self, width=self.WIDTH * self.SCALE_X, height=self.HEIGHT * self.SCALE_Y, bg="white")
        self.canvas.pack()

    def start(self):
        self._draw_info()
        self._draw_floor()
        self.update()

    def update(self):
        self._draw_details()
        self._draw_ship()
        self._draw_result()

    def _draw_info(self):
        self.canvas.delete(self.TAG_INFO)
        self.canvas.create_text(0, 0, anchor=NE, text='auto-lending mode', tags=self.TAG_INFO)

    def _draw_floor(self):
        self.canvas.delete(self.TAG_FLOOR)
        self.canvas.create_line(self.lending.r * self.SCALE_X, self.WIDTH * self.SCALE_Y,
                                self.lending.l * self.SCALE_X, self.HEIGHT * self.SCALE_Y,
                                tags=self.TAG_FLOOR)

    def _draw_details(self):
        self.canvas.delete(self.TAG_DETAILS)
        self.canvas.create_text(0, 0, anchor=NW, text='Fuel: ' + str(self.lending.p), tags=self.TAG_DETAILS)
        self.canvas.create_text(0, 15, anchor=NW, text='Vx: ' + str(self.lending.vx), tags=self.TAG_DETAILS)
        self.canvas.create_text(0, 30, anchor=NW, text='Vy: ' + str(self.lending.vy), tags=self.TAG_DETAILS)
        pass

    def _draw_ship(self):
        self.canvas.delete(self.TAG_SHIP)
        x, y = self.lending.x, -self.lending.y + self.HEIGHT
        self.canvas.create_rectangle((x - 10) * self.SCALE_X, (y - 10) * self.SCALE_Y,
                                     (x + 10) * self.SCALE_X, (y + 10) * self.SCALE_Y,
                                     tags=self.TAG_SHIP)
        if self.lending.last_up:
            self.canvas.create_line(x * self.SCALE_X, (y + 10) * self.SCALE_Y,
                                    x * self.SCALE_X, (y + 15) * self.SCALE_Y,
                                    tags=self.TAG_SHIP)
        if self.lending.last_right:
            self.canvas.create_line((x - 10) * self.SCALE_X, y * self.SCALE_Y,
                                    (x - 15) * self.SCALE_X, y * self.SCALE_Y,
                                    tags=self.TAG_SHIP)
        if self.lending.last_left:
            self.canvas.create_line((x + 10) * self.SCALE_X, y * self.SCALE_Y,
                                    (x + 15) * self.SCALE_X, y * self.SCALE_Y,
                                    tags=self.TAG_SHIP)

    def _draw_result(self):
        self.canvas.delete(self.TAG_RESULT)
        if self.lending.state == 'success':
            self.canvas.create_text(self.WIDTH * self.SCALE_X / 2, self.HEIGHT * self.SCALE_Y / 2,
                                    anchor=CENTER, text='Success!!!', tags=self.TAG_RESULT)
        elif self.lending.state == 'fail':
            self.canvas.create_text(self.WIDTH * self.SCALE_X / 2, self.HEIGHT * self.SCALE_Y / 2,
                                    anchor=CENTER, text='Failed :(', tags=self.TAG_RESULT)
