import sys
import time
from threading import Thread, Event

from auto_lending.lending import Lending
from auto_lending.regulator import ValueRegulator
from auto_lending.lending_drawer import LendingDrawer

url = sys.argv[-2]
max_speed = float(sys.argv[-1])
max_speed_target = max_speed * 0.90

lending = Lending(url)


def target_center():
    return (lending.l + lending.r) / 2


def distance_from_target():
    if lending.x < lending.l + 15:
        return lending.x - (lending.l + 15)
    if lending.x > lending.r - 15:
        return lending.x - (lending.r - 15)
    return 0


def up_speed_target():
    distance = distance_from_target()
    min_height = distance / 2 + 10
    if lending.y < min_height:
        return abs(lending.y - min_height) / 3 if max_speed == 0\
            else max(-max_speed_target, min(max_speed_target, abs(lending.y - min_height) / 3))
    return min(-lending.y / 3.5, -3) if max_speed == 0\
        else max(-max_speed_target, min(max_speed_target, min(-lending.y / 3.5, -3)))

regulator_lr = ValueRegulator(const_p=1, const_i=0, const_d=5,
                              getter_target=lambda: target_center() + lending.vx * (lending.y / lending.vy))
regulator_up = ValueRegulator(const_p=1, const_i=0, const_d=0.5, getter_target=up_speed_target)


def start_drawer() -> LendingDrawer:
    event_started = Event()
    lending_drawer_l = []

    def start_gui():
        lending_drawer = LendingDrawer(lending)
        lending_drawer_l.append(lending_drawer)
        event_started.set()
        lending_drawer.mainloop()

    Thread(target=start_gui, name='GuiThread').start()
    event_started.wait()
    time.sleep(0.5)
    return lending_drawer_l[0]

lending_drawer = start_drawer()

lending.start()
print(lending)
lending_drawer.start()

while lending.state != 'success' and lending.state != 'fail':
    if max_speed != 0 and abs(lending.vx) > max_speed_target:
        if lending.vx > max_speed_target:
            reg_lr_val = -1
        else:
            reg_lr_val = 1
    elif distance_from_target() != 0:
        reg_lr_val = regulator_lr.regulate(lending.x)
    else:
        reg_lr_val = -1 if lending.vx > 2 else 1 if lending.vx < -2 else 0
    reg_up_val = regulator_up.regulate(lending.vy)
    lending.update(up=reg_up_val > 0, left=reg_lr_val < -0.7, right=reg_lr_val > 0.7)
    lending_drawer.update()

    # print(up_speed_target(), lending)
    # land_map = [['.' if bx < lending.l or bx > lending.r else '_' for bx in range(200)]] + [[' ' for x in range(200)] for y in range(200)]
    # land_map[max(0, min(199, int(lending.y)))][max(0, min(199, int(lending.x)))] = 'x'
    # print('\n'.join((''.join(land_map[-y-1]) for y in range(len(land_map)))))

print(lending)
